<?php

namespace ShineSoftware\Setefi\Gateway\Http\Converter;

use Magento\Payment\Gateway\Http\ConverterException;

class Xml implements \Magento\Payment\Gateway\Http\ConverterInterface {

    protected $xml;

    public function convert($source) {
        $this->xml = $this->parserResponse($source);
        if ($this->hasError()) {
            throw new ConverterException(__($this->getErrorMessage()));
        }
        return [
            'paymentid' => $this->getPaymentId(),
            'hostedpageurl' => $this->getHostedPageURL() . "?paymentid=" . $this->getPaymentId(),
            'securitytoken' => $this->getToken()
        ];
    }

    public function getErrorCode() {
        return (string) $this->xml->errorcode;
    }

    public function getErrorMessage() {
        return (string) $this->xml->errormessage;
    }

    private function parserResponse($xml) {
        return simplexml_load_string($xml);
    }

    public function getPaymentId() {
        return (string) $this->xml->paymentid;
    }

    public function getHostedPageURL() {
        return (string) $this->xml->hostedpageurl;
    }

    public function getToken() {
        return (string) $this->xml->securitytoken;
    }

    public function hasError() {
        return !empty((string) $this->xml->errorcode) ? true : false;
    }

    protected function convertCreditCards(\DOMXPath $xpath) {
        $creditCards = [];
        foreach ($xpath->query('/payment/credit_cards/type') as $type) {
            $typeArray = [];
            foreach ($type->childNodes as $typeSubNode) {
                switch ($typeSubNode->nodeName) {
                    case 'label':
                        $typeArray['name'] = $typeSubNode->nodeValue;
                        break;
                    default:
                        break;
                }
            }
            $typeAttributes = $type->attributes;
            $typeArray['order'] = $typeAttributes->getNamedItem('order')->nodeValue;
            $ccId = $typeAttributes->getNamedItem('id')->nodeValue;
            $creditCards[$ccId] = $typeArray;
        }
        uasort($creditCards, [$this, '_compareCcTypes']);
        $config = [];
        foreach ($creditCards as $code => $data) {
            $config[$code] = $data['name'];
        }
        return $config;
    }

    private function _compareCcTypes($left, $right) {
        return $left['order'] - $right['order'];
    }

    protected function convertGroups(\DOMXPath $xpath) {
        $config = [];
        foreach ($xpath->query('/payment/groups/group') as $group) {
            $groupAttributes = $group->attributes;
            $id = $groupAttributes->getNamedItem('id')->nodeValue;
            foreach ($group->childNodes as $groupSubNode) {
                switch ($groupSubNode->nodeName) {
                    case 'label':
                        $config[$id] = $groupSubNode->nodeValue;
                        break;
                    default:
                        break;
                }
            }
        }
        return $config;
    }

    protected function convertMethods(\DOMXPath $xpath) {
        $config = [];
        foreach ($xpath->query('/payment/methods/method') as $method) {
            $name = $method->attributes->getNamedItem('name')->nodeValue;
            foreach ($method->childNodes as $methodSubNode) {
                if ($methodSubNode->nodeType != XML_ELEMENT_NODE) {
                    continue;
                }
                $config[$name][$methodSubNode->nodeName] = $methodSubNode->nodeValue;
            }
        }
        return $config;
    }

}

;
